<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/core
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2024 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Html\Callbacks;

use JchOptimize\Core\Exception\PregErrorException;
use JchOptimize\Core\Html\HtmlElementBuilder;
use JchOptimize\Core\Html\Parser;

class ReduceDom extends \JchOptimize\Core\Html\Callbacks\AbstractCallback
{
    /**
     * @inheritDoc
     */
    public function processMatches(array $matches): string
    {
        if (empty($matches[0])) {
            return $matches[0];
        }
        $attributes = Parser::htmlAttributeWithCaptureValueToken();
        \preg_match("#^<[a-z0-9]++\\b(?:\\s++(?>{$attributes}\\s*+)*?)?\\s*+>#i", $matches[0], $startMatch);
        $startTag = $startMatch[0];
        \preg_match("#</[a-z0-9]++\\s*+>\$#i", $matches[0], $endMatch);
        $endTag = $endMatch[0];
        $content = \substr($matches[0], \strlen($startTag));
        $content = \substr($content, 0, -\strlen($endTag));
        try {
            $element = HtmlElementBuilder::load($startTag . $endTag);
        } catch (PregErrorException) {
            return $matches[0];
        }
        $template = HtmlElementBuilder::template();
        $template->class('jch-reduced-dom-container');
        $template->addChild($content);
        $element->addChild($template);
        return $element->render();
    }
}
